static const char *RcsId = "$Header:  $";
//+=============================================================================
//
// file :         Aml.cpp
//
// description :  C++ source for the Aml and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                Aml are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author:  $
//
// $Revision:  $
//
// $Log:  $
//
// copyleft :     European Synchrotron Radiation Facility
//                BP 220, Grenoble 38043
//                FRANCE
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The following table gives the correspondence
//	between commands and method name.
//
//  Command name        |  Method name
//	----------------------------------------
//  State               |  dev_state()
//  Status              |  dev_status()
//  Axis1_MoveRelative  |  axis1__move_relative()
//  Axis2_MoveRelative  |  axis2__move_relative()
//  Axis1_MoveAbsolute  |  axis1__move_absolute()
//  Axis2_MoveAbsolute  |  axis2__move_absolute()
//  InitMotor           |  init_motor()
//  Stop                |  stop()
//  GetPosition         |  get_position()
//
//===================================================================


#include <tango.h>
#include <Aml.h>
#include <AmlClass.h>

namespace Aml_ns
{

//+----------------------------------------------------------------------------
//
// method : 		Aml::Aml(string &s)
// 
// description : 	constructor for simulated Aml
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
Aml::Aml(Tango::DeviceClass *cl,string &s)
:Tango::Device_4Impl(cl,s.c_str())
{
	device = 0;
	init_device();
}

Aml::Aml(Tango::DeviceClass *cl,const char *s)
:Tango::Device_4Impl(cl,s)
{
	device = 0;
	init_device();
}

Aml::Aml(Tango::DeviceClass *cl,const char *s,const char *d)
:Tango::Device_4Impl(cl,s,d)
{
	device = 0;
	init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		Aml::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void Aml::delete_device()
{
	//	Delete device's allocated object
	if (device > 0) 
		close(device);
	device = 0;

}

//+----------------------------------------------------------------------------
//
// method : 		Aml::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void Aml::init_device()
{
	INFO_STREAM << "Aml::Aml() create device " << device_name << endl;

	// Initialise variables to default values
	//--------------------------------------------
	get_device_property();
	
	initSerialPort();
	init_motor();
	
	Motor1Name = logicalName + axis1_Name;
	Motor2Name = logicalName + axis2_Name;
		
	errorMessage="";
	errorNumber=0;
	
	logMessage="";
}


//+----------------------------------------------------------------------------
//
// method : 		Aml::get_device_property()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void Aml::get_device_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//	Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("LogicalName"));
	dev_prop.push_back(Tango::DbDatum("SerialPort"));
	dev_prop.push_back(Tango::DbDatum("ControllerVersion"));
	dev_prop.push_back(Tango::DbDatum("Axis1_Name"));
	dev_prop.push_back(Tango::DbDatum("Axis1_MinStepRate1"));
	dev_prop.push_back(Tango::DbDatum("Axis1_MinStepRate2"));
	dev_prop.push_back(Tango::DbDatum("Axis1_MinStepRate3"));
	dev_prop.push_back(Tango::DbDatum("Axis1_PhaseCurrent"));
	dev_prop.push_back(Tango::DbDatum("Axis1_Ramp"));
	dev_prop.push_back(Tango::DbDatum("Axis1_StartStopSpeed"));
	dev_prop.push_back(Tango::DbDatum("Axis1_SleewSpeed"));
	dev_prop.push_back(Tango::DbDatum("Axis1_Range"));
	dev_prop.push_back(Tango::DbDatum("Axis1_SetAccelerAndRetardPars"));
	dev_prop.push_back(Tango::DbDatum("Axis2_Name"));
	dev_prop.push_back(Tango::DbDatum("Axis2_MinStepRate1"));
	dev_prop.push_back(Tango::DbDatum("Axis2_MinStepRate2"));
	dev_prop.push_back(Tango::DbDatum("Axis2_MinStepRate3"));
	dev_prop.push_back(Tango::DbDatum("Axis2_PhaseCurrent"));
	dev_prop.push_back(Tango::DbDatum("Axis2_Ramp"));
	dev_prop.push_back(Tango::DbDatum("Axis2_StartStopSpeed"));
	dev_prop.push_back(Tango::DbDatum("Axis2_SleewSpeed"));
	dev_prop.push_back(Tango::DbDatum("Axis2_Range"));
	dev_prop.push_back(Tango::DbDatum("Axis2_SetAccelerAndRetardPars"));

	//	Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_device()->get_property(dev_prop);
	Tango::DbDatum	def_prop, cl_prop;
	AmlClass	*ds_class =
		(static_cast<AmlClass *>(get_device_class()));
	int	i = -1;

	//	Try to initialize LogicalName from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  logicalName;
	else {
		//	Try to initialize LogicalName from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  logicalName;
	}
	//	And try to extract LogicalName value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  logicalName;

	//	Try to initialize SerialPort from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  serialPort;
	else {
		//	Try to initialize SerialPort from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  serialPort;
	}
	//	And try to extract SerialPort value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  serialPort;

	//	Try to initialize ControllerVersion from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  controllerVersion;
	else {
		//	Try to initialize ControllerVersion from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  controllerVersion;
	}
	//	And try to extract ControllerVersion value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  controllerVersion;

	//	Try to initialize Axis1_Name from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis1_Name;
	else {
		//	Try to initialize Axis1_Name from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis1_Name;
	}
	//	And try to extract Axis1_Name value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis1_Name;

	//	Try to initialize Axis1_MinStepRate1 from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis1_MinStepRate1;
	else {
		//	Try to initialize Axis1_MinStepRate1 from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis1_MinStepRate1;
	}
	//	And try to extract Axis1_MinStepRate1 value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis1_MinStepRate1;

	//	Try to initialize Axis1_MinStepRate2 from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis1_MinStepRate2;
	else {
		//	Try to initialize Axis1_MinStepRate2 from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis1_MinStepRate2;
	}
	//	And try to extract Axis1_MinStepRate2 value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis1_MinStepRate2;

	//	Try to initialize Axis1_MinStepRate3 from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis1_MinStepRate3;
	else {
		//	Try to initialize Axis1_MinStepRate3 from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis1_MinStepRate3;
	}
	//	And try to extract Axis1_MinStepRate3 value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis1_MinStepRate3;

	//	Try to initialize Axis1_PhaseCurrent from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis1_PhaseCurrent;
	else {
		//	Try to initialize Axis1_PhaseCurrent from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis1_PhaseCurrent;
	}
	//	And try to extract Axis1_PhaseCurrent value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis1_PhaseCurrent;

	//	Try to initialize Axis1_Ramp from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis1_Ramp;
	else {
		//	Try to initialize Axis1_Ramp from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis1_Ramp;
	}
	//	And try to extract Axis1_Ramp value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis1_Ramp;

	//	Try to initialize Axis1_StartStopSpeed from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis1_StartStopSpeed;
	else {
		//	Try to initialize Axis1_StartStopSpeed from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis1_StartStopSpeed;
	}
	//	And try to extract Axis1_StartStopSpeed value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis1_StartStopSpeed;

	//	Try to initialize Axis1_SleewSpeed from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis1_SleewSpeed;
	else {
		//	Try to initialize Axis1_SleewSpeed from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis1_SleewSpeed;
	}
	//	And try to extract Axis1_SleewSpeed value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis1_SleewSpeed;

	//	Try to initialize Axis1_Range from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis1_Range;
	else {
		//	Try to initialize Axis1_Range from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis1_Range;
	}
	//	And try to extract Axis1_Range value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis1_Range;

	//	Try to initialize Axis1_SetAccelerAndRetardPars from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis1_SetAccelerAndRetardPars;
	else {
		//	Try to initialize Axis1_SetAccelerAndRetardPars from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis1_SetAccelerAndRetardPars;
	}
	//	And try to extract Axis1_SetAccelerAndRetardPars value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis1_SetAccelerAndRetardPars;

	//	Try to initialize Axis2_Name from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis2_Name;
	else {
		//	Try to initialize Axis2_Name from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis2_Name;
	}
	//	And try to extract Axis2_Name value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis2_Name;

	//	Try to initialize Axis2_MinStepRate1 from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis2_MinStepRate1;
	else {
		//	Try to initialize Axis2_MinStepRate1 from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis2_MinStepRate1;
	}
	//	And try to extract Axis2_MinStepRate1 value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis2_MinStepRate1;

	//	Try to initialize Axis2_MinStepRate2 from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis2_MinStepRate2;
	else {
		//	Try to initialize Axis2_MinStepRate2 from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis2_MinStepRate2;
	}
	//	And try to extract Axis2_MinStepRate2 value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis2_MinStepRate2;

	//	Try to initialize Axis2_MinStepRate3 from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis2_MinStepRate3;
	else {
		//	Try to initialize Axis2_MinStepRate3 from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis2_MinStepRate3;
	}
	//	And try to extract Axis2_MinStepRate3 value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis2_MinStepRate3;

	//	Try to initialize Axis2_PhaseCurrent from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis2_PhaseCurrent;
	else {
		//	Try to initialize Axis2_PhaseCurrent from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis2_PhaseCurrent;
	}
	//	And try to extract Axis2_PhaseCurrent value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis2_PhaseCurrent;

	//	Try to initialize Axis2_Ramp from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis2_Ramp;
	else {
		//	Try to initialize Axis2_Ramp from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis2_Ramp;
	}
	//	And try to extract Axis2_Ramp value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis2_Ramp;

	//	Try to initialize Axis2_StartStopSpeed from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis2_StartStopSpeed;
	else {
		//	Try to initialize Axis2_StartStopSpeed from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis2_StartStopSpeed;
	}
	//	And try to extract Axis2_StartStopSpeed value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis2_StartStopSpeed;

	//	Try to initialize Axis2_SleewSpeed from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis2_SleewSpeed;
	else {
		//	Try to initialize Axis2_SleewSpeed from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis2_SleewSpeed;
	}
	//	And try to extract Axis2_SleewSpeed value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis2_SleewSpeed;

	//	Try to initialize Axis2_Range from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis2_Range;
	else {
		//	Try to initialize Axis2_Range from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis2_Range;
	}
	//	And try to extract Axis2_Range value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis2_Range;

	//	Try to initialize Axis2_SetAccelerAndRetardPars from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  axis2_SetAccelerAndRetardPars;
	else {
		//	Try to initialize Axis2_SetAccelerAndRetardPars from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  axis2_SetAccelerAndRetardPars;
	}
	//	And try to extract Axis2_SetAccelerAndRetardPars value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  axis2_SetAccelerAndRetardPars;



	//	End of Automatic code generation
	//------------------------------------------------------------------

}
//+----------------------------------------------------------------------------
//
// method : 		Aml::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void Aml::always_executed_hook()
{
	
}
//+----------------------------------------------------------------------------
//
// method : 		Aml::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void Aml::read_attr_hardware(vector<long> &attr_list)
{
	DEBUG_STREAM << "Aml::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;
	//	Add your own code here
}

//+----------------------------------------------------------------------------
//
// method : 		Aml::read_Axis1_Position
// 
// description : 	Extract real attribute values for Axis1_Position acquisition result.
//
//-----------------------------------------------------------------------------
void Aml::read_Axis1_Position(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Aml::read_Axis1_Position(Tango::Attribute &attr) entering... "<< endl;
	
	attr_long_value=get_position(1);
	currentPosAxis1=attr_long_value;
	
	attr.set_value(&attr_long_value);

}

//+----------------------------------------------------------------------------
//
// method : 		Aml::read_Axis2_Position
// 
// description : 	Extract real attribute values for Axis2_Position acquisition result.
//
//-----------------------------------------------------------------------------
void Aml::read_Axis2_Position(Tango::Attribute &attr)
{
	DEBUG_STREAM << "Aml::read_Axis2_Position(Tango::Attribute &attr) entering... "<< endl;
	
	attr_long_value=get_position(2);
	currentPosAxis2=attr_long_value;
	
	attr.set_value(&attr_long_value);
}


//+------------------------------------------------------------------
/**
 *	method:	Aml::init_motor
 *
 *	description:	method to execute "InitMotor"
 *
 *
 */
//+------------------------------------------------------------------
void Aml::init_motor()
{
	DEBUG_STREAM << "Aml::init_motor(): entering... !" << endl;

	//	Add your own code to control device here
	char *answ;
	string cmd;
	string selMot;
	char *char_cmd;

	actualSelectedMotor = -1;
	//select 1st axis
	selMot=string(AML_AXIS1);
	selectMotor(selMot);
	
	
	//set 1st axis parameters
	if (device != 0) {
		if (controllerVersion)
			cmd=axis1_MinStepRate1;
		else
			cmd=string(AML_SET_MIN_STEP_RATE1) + "," + axis1_MinStepRate1;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor1Name<<" init_motor: setting min step rate1 failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if ((device != 0) && (!controllerVersion)) {
		cmd=string(AML_SET_MIN_STEP_RATE2) + "," + axis1_MinStepRate2;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor1Name<<" init_motor: setting min step rate2 failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if ((device != 0) && (!controllerVersion)) {
		cmd=string(AML_SET_MIN_STEP_RATE4) + "," + axis1_MinStepRate3;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor1Name<<" init_motor: setting min step rate4 failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if (device != 0) {
		cmd=string(AML_REMOVE_PHASE_CURRENT) + axis1_PhaseCurrent;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor1Name<<" init_motor: setting phase current failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if ((device != 0) && (!controllerVersion)) {
		cmd=string(AML_SET_RAMP) + axis1_Ramp;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor1Name<<" init_motor: setting ramp failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if (device != 0) {
		cmd=string(AML_SMOOTH_STOP);
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor1Name<<" init_motor: setting smooth stop failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if ((device != 0) && (!controllerVersion)) {
		cmd=string(AML_START_STOP_SPEED) + axis1_StartStopSpeed;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor1Name<<" init_motor: setting start stop speed failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if ((device != 0) && (!controllerVersion)) {
		cmd=string(AML_SET_SLEEW_SPEED) + axis1_SleewSpeed;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor1Name<<" init_motor: setting sleew speed failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if (device != 0) {
		cmd=string(AML_SET_INIT_POS) ;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor1Name<<" init_motor: setting init pos failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if ((device != 0) && (controllerVersion)) {
		cmd=axis1_SetAccelerAndRetardPars;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor1Name<<" init_motor: setting sleew speed failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}
	
	//select 2nd axis
	selMot=string(AML_AXIS2);
	selectMotor(selMot);
	
	//set 2nd axis parameters
	if (device != 0) {
		if (controllerVersion)
			cmd=axis2_MinStepRate1;
		else
			cmd=string(AML_SET_MIN_STEP_RATE1) + "," + axis2_MinStepRate1;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor2Name<<" init_motor: setting min step rate1 failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if ((device != 0) && (!controllerVersion)) {
		cmd=string(AML_SET_MIN_STEP_RATE2) + "," + axis2_MinStepRate2;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor2Name<<" init_motor: setting min step rate2 failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if ((device != 0) && (!controllerVersion)) {
		cmd=string(AML_SET_MIN_STEP_RATE4) + "," + axis2_MinStepRate3;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor2Name<<" init_motor: setting min step rate4 failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if (device != 0) {
		cmd=string(AML_REMOVE_PHASE_CURRENT) + axis2_PhaseCurrent;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor2Name<<" init_motor: setting phase current failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if ((device != 0) && (!controllerVersion)) {
		cmd=string(AML_SET_RAMP) + axis2_Ramp;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor2Name<<" init_motor: setting ramp failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if (device != 0) {
		cmd=string(AML_SMOOTH_STOP);
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor2Name<<" init_motor: setting smooth stop failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if ((device != 0) && (!controllerVersion)) {
		cmd=string(AML_START_STOP_SPEED) + axis2_StartStopSpeed;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor2Name<<" init_motor: setting start stop speed failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if ((device != 0) && (!controllerVersion)) {
		cmd=string(AML_SET_SLEEW_SPEED) + axis2_SleewSpeed;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor2Name<<" init_motor: setting sleew speed failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if (device != 0) {
		cmd=string(AML_SET_INIT_POS) ;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor2Name<<" init_motor: setting init pos failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	if ((device != 0) && (controllerVersion)) {
		cmd=axis2_SetAccelerAndRetardPars;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< Motor2Name<<" init_motor: setting sleew speed failed";
			Tango::Except::throw_exception("Init Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}

	return;

}



//+------------------------------------------------------------------
/**
 *			General methods
 */
//+------------------------------------------------------------------
void Aml::initSerialPort()
{
	DEBUG_STREAM << "Aml::initSerialPort(): entering... !" << endl;

	//	Add your own code to control device here

	struct termios tty_settings;
	struct termios com_par;
	char ttyport[128];


	// Initialise variables to default values
	//--------------------------------------------
	sprintf(ttyport,"/dev/%s",serialPort.c_str());
	device = open (ttyport, O_RDWR | O_NONBLOCK);
	if (device < 0) {
	       device = 0;
		Tango::Except::throw_exception(
			"Unable to open serial device",
			"AML Init Error.",
			"initSerialPort()",
			Tango::ERR);
		return;
	}

	if(tcgetattr(device, &com_par) == -1) {
	       close(device);
	       device = 0;
	       Tango::Except::throw_exception(
		       "Unable to get serial device attributes",
		       "AML Init Error.",
		       "initSerialPort()",
		       Tango::ERR);
	       return;
	}
	
	bzero(&tty_settings, sizeof(tty_settings));

	tty_settings.c_oflag = 0; // raw output
	tty_settings.c_iflag = IGNBRK | IGNPAR | ICRNL; 
	tty_settings.c_cflag = B19200 | CS7 | CSTOPB | PARODD | PARENB | CLOCAL | CREAD;

	//timeout della seriale
	tty_settings.c_cc[VMIN]=1; 		/* blocking read until N character arrives */
	tty_settings.c_cc[VTIME]=0;  		/* inter-character timer  */

	cfsetospeed(&tty_settings,B19200);   	// input baud rate
	cfsetispeed(&tty_settings,B19200);      // output


	if (tcflush(device,TCIOFLUSH)!=0)
	{
		close(device);
		device = 0;
		Tango::Except::throw_exception(
			"Unable to flush serial device",
			"AML int Error.",
			"init_serialPort()",
			Tango::ERR);
			return;
	}

       if(tcsetattr(device, TCSANOW, &tty_settings) == -1) {
		close(device);
		device = 0;
		Tango::Except::throw_exception(
			"tcsetattr of serial device failed!!!",
			"AML int Error.",
			"init_serialPort()",
			Tango::ERR);
			return;
       }			

	if (tcflush(device,TCIOFLUSH)!=0)
	{
		close(device);
		device = 0;
		Tango::Except::throw_exception(
			"Unable to flush serial device after having set tty parameters",
			"AML int Error.",
			"init_serialPort()",
			Tango::ERR);
			return;
	}

	tcsetattr(device, TCSANOW,&tty_settings);// changes made NOW

	//flusho la seriale
	if (tcflush(device,TCIOFLUSH)!=0)
	{
		Tango::Except::throw_exception(
			"Unable to flush serial device",
			"AML int Error.",
			"init_serialPort()",
			Tango::ERR);
			return;
	}

}


bool Aml:: checkRangeOk(string axis,Tango::DevLong distance) 
{
	Tango::DevLong Range=0;
	bool res=false;
	if (strcmp(axis.c_str(), AML_AXIS1)== 0)
		Range= atol(axis1_Range.c_str());
	else if (strcmp(axis.c_str(), AML_AXIS2)== 0)
		Range= atol(axis2_Range.c_str());
	else 
		Tango::Except::throw_exception(
			"NO AXIS SELECTED",
			"AML setup error.",
			"checkRangeOk()",
			Tango::ERR);
	res = 	((distance> -Range) && (distance<Range));
	return res;
}

void Aml::ClearError(void) {
  errorMessage.clear();
  errorNumber = 0;
}

void Aml::SetError(char *msg, int num = 0) {
 ClearError();
 errorNumber = num; 
 errorMessage = string(msg);
}


void Aml::amlError(char *msg, char *answ) {
 char buffer[AMLBUFFLEN];

 sprintf(buffer,"%s: %s\n",logicalName.c_str(),msg);

 if (!(strcmp(answ,"E1")))
  sprintf(buffer+strlen(buffer),"Parity/Checksum error");
 if (!(strcmp(answ,"E2")))
  sprintf(buffer+strlen(buffer),"Command argument out of limit or not requested");
 if (!(strcmp(answ,"E3")))
  sprintf(buffer+strlen(buffer),"Error in downloaded program");
 if (!(strcmp(answ,"E4")))
  sprintf(buffer+strlen(buffer),"Command  not executable");
 if (!(strcmp(answ,"E5")))
  sprintf(buffer+strlen(buffer),"Temperature exceeded 175 C");
 if (!(strcmp(answ,"E7")))
  sprintf(buffer+strlen(buffer),"End of travel input at \"low\"");
 if (!(strcmp(answ,"E8")))
  sprintf(buffer+strlen(buffer),"Loop nesting too deep");
 if (!(strcmp(answ,"E9")))
  sprintf(buffer+strlen(buffer),"Program too large");

 errorMessage=string(buffer);
}


char * Aml::talkListen(char *msg, const int retry) {
 int readmask = 0;
 fd_set readfds;
 int nfound, nbytes, total_read_bytes,i;
 char errbuffer[AMLBUFFLEN],logbuffer[AMLBUFFLEN];
 char *buffer;
 char answ_buffer[AMLBUFFLEN];
 struct timeval timeout;

 tcflush(device,TCIOFLUSH);
 DEBUG_STREAM << "Aml::talkListen(): Sending: " << msg << endl;
 buffer = strdup(msg);
 buffer[strlen(msg)] = '\r';
 if ( (nbytes = write(device, buffer, strlen(msg)+1)) < 0) {
  sprintf(errbuffer,"%s: talk_listen: write failed: errno=%d",logicalName.c_str(), msg, errno); 
  SetError(errbuffer,errno);
  return(0);
 }
 usleep(1000);
 free(buffer);
 readmask = 1 << device;

 bzero(answ_buffer,AMLBUFFLEN);
 total_read_bytes = 0;

 for (i=0; i<retry; i++) {

// changes required

   FD_ZERO(&readfds);
   FD_SET(device, &readfds);
   timeout.tv_sec = 1;
   timeout.tv_usec = 0;
   
   if((nfound = select(32, &readfds, 0, 0, &timeout)) == -1) {
     sprintf(errbuffer,"%s: talk_listen: select failed",logicalName.c_str(), msg); 
     SetError(errbuffer,nfound);
     return(0);
     }
   else if (nfound > 0) {
       if ( (nbytes = read(device, answ_buffer+total_read_bytes, AMLBUFFLEN)) < 0) {
	  sprintf(errbuffer,"%s: talk_listen: read failed: errno=%d",logicalName.c_str(), msg, errno); 
  	  SetError(errbuffer,errno);
	  return(0);
       }
       total_read_bytes += nbytes;
       if (strchr(answ_buffer,'\n') == NULL) {
       	continue;
       }
       *(strchr(answ_buffer,'\n')) = 0;
       DEBUG_STREAM << "Aml::talkListen(): Received: " << answ_buffer << endl;
       if(!strncmp(answ_buffer,AMLOKY,1) ||
	  !strncmp(answ_buffer,AMLOKV,1) || 
	  !strncmp(answ_buffer,AMLOKQ,1) ) {
     	   return(strdup(answ_buffer));
       } else
       if (!strncmp(answ_buffer,AMLBUSY,1)) {
	 sprintf(logbuffer,"\n%s talk_listen: answer:|%s|: busy",logicalName.c_str(),answ_buffer); 
	 logMessage=string(logbuffer);
       } else
       if (!strncmp(answ_buffer,AMLERROR,1)) {
	 sprintf(errbuffer,"%s talk_listen: answer:|%s|: error",logicalName.c_str(),answ_buffer); 
	 amlError(errbuffer, answ_buffer);
	 return(0);

       } else {
	 sprintf(errbuffer,"%s talk_listen: answer:|%s|: unknown",logicalName.c_str(),answ_buffer); 
	 SetError(errbuffer,1);
	 return(0);
       } 
   }
   usleep(1000);
 }
 sprintf(errbuffer,"%s talk_listen: answer:|%s|: select timeout",logicalName.c_str(),answ_buffer); 
 SetError(errbuffer,1);
 return(0);
}

int Aml:: checkStatus() 
{
 char buffer[256];
 char *answ;

 if (device == 0) {
  	sprintf(buffer,"%s: getstatus: serial not initialized",logicalName.c_str());
	DEBUG_STREAM << buffer << endl;
	return -1;
	}
 
 answ=talkListen("F", AML_MAX_RETRY);	
 if (answ==0) {
      return -1;
 }
 sprintf(buffer,"%s: getstatus: motor %s",logicalName.c_str(), answ);
 DEBUG_STREAM << buffer << endl;

 while (answ[0] == 'B') {
  sprintf(buffer,"%s: getstatus: motor %s",logicalName.c_str(), answ);
  DEBUG_STREAM << buffer << endl;
  free(answ);
  answ=talkListen("F", AML_MAX_RETRY);	
     if (answ==0) {
     	return -1;
   }
 }

 sprintf(buffer,"%s: getstatus: motor %s",logicalName.c_str(), answ);
 DEBUG_STREAM << buffer << endl;
 if (answ[0] == 'Y') {
    // motor is ready
    free(answ);
    return(0);
 } else {
    // motor in error
    free(answ);
    return -1;
 }
}

void Aml:: selectMotor(string selectedMotor) 
{
	char *answ;
	string cmd;
	char *char_cmd;
	int motnum = atoi(selectedMotor.c_str());

	if ((device != 0) && ( actualSelectedMotor != motnum)){
		cmd=string(AML_SEL_MOTOR) + selectedMotor;
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< logicalName + selectedMotor <<" selectMotor: selection of motor failed";
			Tango::Except::throw_exception("Select Motor Error", os.str(), __func__);
 		} else {
			actualSelectedMotor = motnum;
			// Sleep a little bit before next operation
			usleep(1000);
		}
		free(answ);
		delete []char_cmd;
	}
	return;
}


//+------------------------------------------------------------------
/**
 *	method:	Aml::get_position
 *
 *	description:	method to execute "GetPosition"
 *
 * @return	
 *
 */
//+------------------------------------------------------------------
//Tango::DevFloat Aml::get_position()
//{
//       Tango::DevFloat	argout ;
//       DEBUG_STREAM << "Aml::get_position(): entering... !" << endl;
//
//       selectMotor();
//
//       char *recbuffer;
//       char logbuffer[256];
//       string cmd;
//       char *char_cmd;
//
//       if (device != 0) {
//	  cmd=string(AML_GET_POS) ;
//	  char_cmd=new char[cmd.length()+1];
//	  strcpy(char_cmd, cmd.c_str());
//
//	  recbuffer = talkListen(char_cmd, AML_MAX_RETRY);;
//	  if ( recbuffer== NULL) {
//		TangoSys_OMemStream os;
//		os << "Error->"<< logicalName<<" get_position: get pos failed";
//		Tango::Except::throw_exception("Aml::get_position(): Error", os.str(), __func__);
// 	  }
//		delete []char_cmd;
//	  }
//  
//
//       argout=atof(recbuffer+1);
//       free(recbuffer);
//       return argout;
//}

//+------------------------------------------------------------------
/**
 *	method:	Aml::get_position
 *
 *	description:	method to execute "GetPosition"
 *
 * @param	argin	
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevLong Aml::get_position(Tango::DevShort argin)
{
	Tango::DevLong	argout ;
	DEBUG_STREAM << "Aml::get_position(): entering... !" << endl;

	//	Add your own code to control device here

	string selectedMotor;
	std::ostringstream oss;	
	oss<<argin;

       selectedMotor=oss.str();
       selectMotor(selectedMotor);

       char *recbuffer;
       char logbuffer[256];
       string cmd;
       char *char_cmd;

       if (device != 0) {
	  cmd=string(AML_GET_POS) ;
	  char_cmd=new char[cmd.length()+1];
	  strcpy(char_cmd, cmd.c_str());

	  recbuffer = talkListen(char_cmd, AML_MAX_RETRY);;
	  if ( recbuffer== NULL) {
		TangoSys_OMemStream os;
		os << "Error->"<< logicalName<<" get_position: get pos failed";
		Tango::Except::throw_exception("Aml::get_position(): Error", os.str(), __func__);
 	  }
		delete []char_cmd;
	  }
  

       argout=atol(recbuffer+1);
       free(recbuffer);
       return argout;
	
}




//+------------------------------------------------------------------
/**
 *	method:	Aml::axis1__move_relative
 *
 *	description:	method to execute "Axis1_MoveRelative"
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void Aml::axis1__move_relative(Tango::DevLong argin)
{
	DEBUG_STREAM << "Aml::axis1__move_relative(): entering... !" << endl;

	//	Add your own code to control device here
	string selectedAxis=string(AML_AXIS1);
	selectMotor(selectedAxis);
	
	char *answ;
	
	if (checkStatus()<0) {
		TangoSys_OMemStream os;
		os << "Error->"<< logicalName + selectedAxis<<" move_relative: motor NOT ready";
		Tango::Except::throw_exception("Motor Busy Error", os.str(), __func__);
		return;
	}
	
	currentPosAxis1=get_position(atol(selectedAxis.c_str()));
	Tango::DevLong dist = currentPosAxis1 + argin;	
	
	if (!checkRangeOk(selectedAxis, dist)) {
		TangoSys_OMemStream os;
		os << "Error->"<< logicalName + selectedAxis<<" move_relative: out of range";
		Tango::Except::throw_exception("Moving Error", os.str(), __func__);
		return;
	}

	if (device != 0) {
		char cmd[AMLBUFFLEN];
		sprintf(cmd,"%s%d",AML_ABS_POS,(int)dist);
		DEBUG_STREAM << dist << ","<< currentPosAxis1 << "," << argin << "," << AML_ABS_POS << "," << cmd << endl;
		answ=talkListen(cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< logicalName + selectedAxis<<" move_relative: movement failed";
			Tango::Except::throw_exception("Motor Error", os.str(), __func__);
 		}
		free(answ);
	}
	return;

}

//+------------------------------------------------------------------
/**
 *	method:	Aml::axis2__move_relative
 *
 *	description:	method to execute "Axis2_MoveRelative"
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void Aml::axis2__move_relative(Tango::DevLong argin)
{
	DEBUG_STREAM << "Aml::axis2__move_relative(): entering... !" << endl;

	//	Add your own code to control device here
	string selectedAxis=string(AML_AXIS2);

	selectMotor(selectedAxis);
	
	char *answ;
	
	if (checkStatus()<0) {
		TangoSys_OMemStream os;
		os << "Error->"<< logicalName + selectedAxis<<" move_relative: motor NOT ready";
		Tango::Except::throw_exception("Motor Busy Error", os.str(), __func__);
		return;
	}
	
	currentPosAxis2=get_position(atol(selectedAxis.c_str()));
	Tango::DevLong dist = currentPosAxis2 + argin;	
	if (!checkRangeOk(selectedAxis, dist)) {
		TangoSys_OMemStream os;
		os << "Error->"<< logicalName + selectedAxis<<" move_relative: out of range";
		Tango::Except::throw_exception("Moving Error", os.str(), __func__);
		return;
	}

	if (device != 0) {
		char cmd[AMLBUFFLEN];
		sprintf(cmd,"%s%d",AML_ABS_POS,(int)dist);
		answ=talkListen(cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< logicalName + selectedAxis<<" move_relative: movement failed";
			Tango::Except::throw_exception("Motor Error", os.str(), __func__);
 		}
		free(answ);
	}
	return;

}

//+------------------------------------------------------------------
/**
 *	method:	Aml::axis1__move_absolute
 *
 *	description:	method to execute "Axis1_MoveAbsolute"
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void Aml::axis1__move_absolute(Tango::DevLong argin)
{
	DEBUG_STREAM << "Aml::axis1__move_absolute(): entering... !" << endl;

	//	Add your own code to control device here

	string selectedAxis=string(AML_AXIS1);
	
	selectMotor(selectedAxis);      
      
       char buffer[256], logbuffer[256];
       char *answ;
       
	Tango::DevLong dist = argin;	


       if (!checkRangeOk(selectedAxis,argin)) {
		TangoSys_OMemStream os;
		os << "Error->"<< logicalName + selectedAxis<<" move_absolute: out of range";
		Tango::Except::throw_exception("Moving Error", os.str(), __func__);
		return;
       }

      //  control the motor state
       if ( checkStatus() < 0) {  // motor not ready (e.i. busy or in error)
		TangoSys_OMemStream os;
		os << "Error->"<< logicalName +  selectedAxis<<" move_absolute: motor NOT ready";
		Tango::Except::throw_exception("Motor Busy Error", os.str(), __func__);
		return;
      }

	if (device != 0) {
		char cmd[AMLBUFFLEN];
		sprintf(cmd,"%s%d",AML_ABS_POS,(int)dist);
		answ=talkListen(cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< logicalName +  selectedAxis<<" move_absolute: movement failed";
			Tango::Except::throw_exception("Motor Error", os.str(), __func__);
 		}
		free(answ);
	}

       return;


}

//+------------------------------------------------------------------
/**
 *	method:	Aml::axis2__move_absolute
 *
 *	description:	method to execute "Axis2_MoveAbsolute"
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void Aml::axis2__move_absolute(Tango::DevLong argin)
{
	DEBUG_STREAM << "Aml::axis2__move_absolute(): entering... !" << endl;

	//	Add your own code to control device here
	string selectedAxis=string(AML_AXIS2);
	selectMotor(selectedAxis);      
      
       char buffer[256], logbuffer[256];
       char *answ;
       
	Tango::DevLong dist = argin;	

       if (!checkRangeOk(selectedAxis,argin)) {
		TangoSys_OMemStream os;
		os << "Error->"<< logicalName + selectedAxis<<" move_absolute: out of range";
		Tango::Except::throw_exception("Moving Error", os.str(), __func__);
		return;
       }

      //  controll the motor state
       if ( checkStatus() < 0) {  // motor not ready (e.i. busy or in error)
		TangoSys_OMemStream os;
		os << "Error->"<< logicalName +  selectedAxis<<" move_absolute: motor NOT ready";
		Tango::Except::throw_exception("Motor Busy Error", os.str(), __func__);
		return;
      }

	if (device != 0) {
		char cmd[AMLBUFFLEN];
		sprintf(cmd,"%s%d",AML_ABS_POS,(int)dist);
		answ=talkListen(cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< logicalName + selectedAxis<<" move_absolute: movement failed";
			Tango::Except::throw_exception("Motor Error", os.str(), __func__);
 		}
		free(answ);
	}

       return;

}

//+------------------------------------------------------------------
/**
 *	method:	Aml::stop
 *
 *	description:	method to execute "Stop"
 *
 * @param	argin	
 *
 */
//+------------------------------------------------------------------
void Aml::stop(Tango::DevShort argin)
{
	DEBUG_STREAM << "Aml::stop(): entering... !" << endl;

	//	Add your own code to control device here

	string selectedMotor;
	std::ostringstream oss;	
	oss<<argin;

	selectedMotor=oss.str();
       selectMotor(selectedMotor);
	
	char *answ;
	string cmd;
	char *char_cmd;

	if (device != 0) {
		cmd=string(AML_SMOOTH_STOP);
		char_cmd=new char[cmd.length()+1];
		strcpy(char_cmd, cmd.c_str());
		answ=talkListen(char_cmd, AML_MAX_RETRY);
		if (answ == 0) {
			TangoSys_OMemStream os;
			os << "Error->"<< logicalName + selectedMotor <<" stop: stopping of motor failed";
			Tango::Except::throw_exception("Stop Motor Error", os.str(), __func__);
 		}
		free(answ);
		delete []char_cmd;
	}
	
	return;

}



}	//	namespace
